/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <pan/general/foreach.h>
#include <pan/general/i18n.h>
#include <pan/tasks/queue.h>
#include <pan/tasks/task-xover.h>
#include "dl-headers-ui.h"
#include "pad.h"

using namespace pan;

namespace
{
  struct State
  {
    Data& data;
    Queue& queue;
    quarks_t groups;
    GtkWidget * dialog;
    GtkWidget * all_headers_rb;
    GtkWidget * new_headers_rb;
    GtkWidget * latest_n_headers_rb;
    GtkWidget * n_headers_spinbutton;
    State (Data& d, Queue& q): data(d), queue(q) {}
  };

  void delete_state (gpointer state_gpointer)
  {
    delete static_cast<State*>(state_gpointer);
  }

  void response_cb (GtkDialog * dialog, int response, gpointer user_data)
  {
    State * state (static_cast<State*>(user_data));

    if (response == GTK_RESPONSE_ACCEPT) {
      foreach_const (quarks_t, state->groups, it) {
        Task * task;
        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(state->all_headers_rb)))
          task = new TaskXOver (state->data, *it, TaskXOver::ALL);
        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(state->new_headers_rb)))
          task = new TaskXOver (state->data, *it, TaskXOver::NEW);
        else { /* N headers */
          const int sample_size (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(state->n_headers_spinbutton)));
          task = new TaskXOver (state->data, *it, TaskXOver::SAMPLE, sample_size);
        }
        state->queue.add_task (task);
      }
    }

    gtk_widget_destroy (GTK_WIDGET(dialog));
  }

  int
  spin_tickled_cb (GtkWidget * w, gpointer * event, gpointer user_data)
  {
    // if a user clicked in the spinbutton window,
    // select the spinbutton radiobutton for them.
    State * state (static_cast<State*>(user_data));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(state->latest_n_headers_rb), true);
    return false;
  }
}

void
pan :: headers_dialog (Data& data, Queue& queue, const quarks_t& groups, GtkWindow * parent)
{
  if (!groups.empty())
  {
    State * state = new State (data, queue);
    state->groups = groups;
    state->dialog = gtk_dialog_new_with_buttons (
      _("Pan: Get Headers from Server"), parent, GTK_DIALOG_DESTROY_WITH_PARENT,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
      GTK_STOCK_EXECUTE, GTK_RESPONSE_ACCEPT,
      NULL);
    gtk_dialog_set_default_response (GTK_DIALOG(state->dialog), GTK_RESPONSE_ACCEPT);

    GtkWidget * w;
    GtkWidget * vbox = gtk_vbox_new (FALSE, PAD_SMALL);
    gtk_container_set_border_width (GTK_CONTAINER(vbox), 18);
    state->new_headers_rb = w = gtk_radio_button_new_with_mnemonic (NULL, _("Get _new headers"));
    gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
    state->all_headers_rb = w = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON(w), _("Get _all headers"));
    gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
    GtkWidget * hbox = gtk_hbox_new (FALSE, 0);
    state->latest_n_headers_rb = w = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON(w), _("Get _recent headers: "));
    gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
    GtkAdjustment * adj = GTK_ADJUSTMENT(gtk_adjustment_new (150, 0, INT_MAX, 50, 50, 1));
    state->n_headers_spinbutton = w = gtk_spin_button_new (adj, 1, 0);
    gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    gtk_container_add (GTK_CONTAINER(GTK_DIALOG(state->dialog)->vbox), vbox);
    g_signal_connect (w, "focus_in_event", GTK_SIGNAL_FUNC(spin_tickled_cb), state);
    g_signal_connect (w, "button_press_event", GTK_SIGNAL_FUNC(spin_tickled_cb), state);
    g_signal_connect (state->dialog, "response", G_CALLBACK(response_cb), state);
    g_object_set_data_full (G_OBJECT(state->dialog), "state", state, delete_state);
    gtk_widget_show_all (state->dialog);
  }
}
