/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __NNTP_Pool_h__
#define __NNTP_Pool_h__

#include <deque>
#include <string>
#include <pan/general/quark.h>
#include <pan/tasks/socket.h>
#include <pan/tasks/nntp.h>

namespace pan
{
  /**
   * A pool of NNTP connections to a particular server.
   *
   * @ingroup tasks 
   */
   class NNTP_Pool: private NNTP::Listener, public NNTP::Source
   {
      public:

         NNTP_Pool (const Quark       & server,
                    Socket::Creator   *,
                    const std::string & username,
                    const std::string & password);

         virtual ~NNTP_Pool ();

         void set_address (const StringView& host, int port);
         void set_auth (const StringView& username, const StringView& password);
         void set_max_connections (int connections);
         virtual void check_in (NNTP*, bool is_ok);
         NNTP* check_out ();
         void set_online (bool);
         bool is_online () const;
         void get_counts (int& setme_active, int& setme_idle, int& setme_connecting, int& setme_max) const;

      public:

         /** Interface class for objects that listen to an NNTP_Pool's events */
         class Listener {
            public:
               virtual ~Listener () { };
               virtual void on_nntp_available (const Quark& server) = 0;
         };

         void add_listener (Listener * l);
         void remove_listener (Listener * l);
         void request_nntp ();

      private:

         friend class NNTP;
         virtual void on_nntp_done (NNTP*, Health);

      private:

         void fire_nntp_available ();

         const Quark _server;
         Socket::Creator * _socket_creator;
         bool _is_online;
         bool _connection_pending;
         struct PoolItem {
            NNTP * nntp;
            bool is_checked_in;
            time_t last_active_time;
         };
         typedef std::deque<PoolItem> pool_items_t;
         pool_items_t _pool_items;

         std::string _host;
         int _port;
         std::string _username;
         std::string _password;
         int _max_connections;

         typedef std::set<Listener*> listeners_t;
         listeners_t _listeners;
   };
};

#endif // __NNTP_Pool_h__
